@@ -28,13 +28,12 @@ pointing at, rather than what it might appear to be pointing at.
 =cut
  
 use strict;
-use base 'PPI::Token';
-use Params::Util '_INSTANCE';
+use Params::Util qw{_INSTANCE};
+use PPI::Token   ();
 
-use vars qw{$VERSION};
-BEGIN {
-	$VERSION = '1.203';
-}
+our $VERSION = '1.277';
+
+our @ISA = "PPI::Token";
 
 
 
@@ -60,8 +59,8 @@ variations.
 sub canonical {
 	my $symbol = shift->content;
 	$symbol =~ s/\s+//;
-	$symbol =~ s/(?<=[\$\@\%\&\*])::/main::/;
 	$symbol =~ s/\'/::/g;
+	$symbol =~ s/(?<=[\$\@\%\&\*])::/main::/;
 	$symbol;
 }
 
@@ -81,29 +80,41 @@ Returns the symbol as a string.
 
 =cut
 
+my %cast_which_trumps_braces = map { $_ => 1 } qw{ $ @ % };
+
 sub symbol {
 	my $self   = shift;
 	my $symbol = $self->canonical;
 
 	# Immediately return the cases where it can't be anything else
 	my $type = substr( $symbol, 0, 1 );
-	return $symbol if $type eq '%';
 	return $symbol if $type eq '&';
 
 	# Unless the next significant Element is a structure, it's correct.
 	my $after  = $self->snext_sibling;
 	return $symbol unless _INSTANCE($after, 'PPI::Structure');
 
-	# Process the rest for cases where it might actually be somethign else
+	# Process the rest for cases where it might actually be something else
 	my $braces = $after->braces;
 	return $symbol unless defined $braces;
 	if ( $type eq '$' ) {
+
+		# If it is cast to '$' or '@', that trumps any braces
+		my $before = $self->sprevious_sibling;
+		return $symbol if $before &&
+			$before->isa( 'PPI::Token::Cast' ) &&
+			$cast_which_trumps_braces{ $before->content };
+
+		# Otherwise the braces rule
 		substr( $symbol, 0, 1, '@' ) if $braces eq '[]';
 		substr( $symbol, 0, 1, '%' ) if $braces eq '{}';
 
 	} elsif ( $type eq '@' ) {
 		substr( $symbol, 0, 1, '%' ) if $braces eq '{}';
 
+	} elsif ( $type eq '%' ) {
+		substr( $symbol, 0, 1, '@' ) if $braces eq '[]';
+
 	}
 
 	$symbol;
@@ -121,8 +132,7 @@ Returns the sigil as a string.
 =cut
 
 sub raw_type {
-	my $self = shift;
-	substr( $self->content, 0, 1 );
+	substr( $_[0]->content, 0, 1 );
 }
 
 =pod
@@ -137,8 +147,7 @@ Returns the sigil as a string.
 =cut
 
 sub symbol_type {
-	my $self = shift;
-	substr( $self->symbol, 0, 1 );
+	substr( $_[0]->symbol, 0, 1 );
 }
 
 
@@ -152,8 +161,8 @@ sub __TOKENIZER__on_char {
 	my $t = $_[1];
 
 	# Suck in till the end of the symbol
-	my $line = substr( $t->{line}, $t->{line_cursor} );
-	if ( $line =~ /^([\w:']+)/ ) {
+	pos $t->{line} = $t->{line_cursor};
+	if ( $t->{line} =~ m/\G([\w:\']+)/gc ) {
 		$t->{token}->{content} .= $1;
 		$t->{line_cursor}      += length $1;
 	}
@@ -190,7 +199,7 @@ sub __TOKENIZER__on_char {
 	# Trim off anything we oversucked...
 	$content =~ /^(
 		[\$@%&*]
-		(?: : (?!:) | # Allow single-colon non-magic vars
+		(?: : (?!:) | # Allow single-colon non-magic variables
 			(?: \w+ | \' (?!\d) \w+ | \:: \w+ )
 			(?:
 				# Allow both :: and ' in namespace separators
@@ -221,7 +230,7 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2001 - 2008 Adam Kennedy.
+Copyright 2001 - 2011 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
